module Controle(

input clk,
input rst,
input [5:0] Opcode,
input [5:0] Funct,
input DivZero,
input LT,
input GT,
input ET,
input O,
input ParaDiv,
output reg [2:0] PcSel,
output reg [1:0] Iord,
output reg [1:0] StoreSel,
output reg [1:0] LoadSel,
output reg [1:0] ScrReg,
output reg [2:0] ScrData,
output reg [2:0] ShiftSel,
output reg [1:0] ScrA,
output reg [1:0] ScrB,
output reg [2:0] AluOp,
output reg PcWrite,
output reg EpcWrite,
output reg MemStatus,
output reg IrWrite,
output reg ScrShift,
output reg RegWrite,
output reg SignSel,
output reg DivSel,
output reg DivStart,
output reg rstDiv,
output reg LoadReg,
output reg [5:0] TestadoAtual
);

parameter [5:0] Reset = 6'd1;
parameter [5:0] Busca = 6'd2;
parameter [5:0] Wait_Ir = 6'd3;
parameter [5:0] Calc_Branch = 6'd4;
parameter [5:0] Rte = 6'd5;
parameter [5:0] Jump = 6'd6;
parameter [5:0] Branch = 6'd7;
parameter [5:0] Lui = 6'd8;
parameter [5:0] Jr = 6'd9;
parameter [5:0] OpIne = 6'd10;
parameter [5:0]	Div = 6'd11;
parameter [5:0] Sll = 6'd12;
parameter [5:0] Sra = 6'd13;
parameter [5:0] Srl = 6'd14;
parameter [5:0] Sllv = 6'd15;
parameter [5:0] Srav = 6'd16;
parameter [5:0] Break = 6'd17;
parameter [5:0] Mfhi = 6'd18;
parameter [5:0] Mflo = 6'd19;
parameter [5:0] Slt = 6'd20;
parameter [5:0] Slti = 6'd21;
parameter [5:0] Jal = 6'd22;
parameter [5:0] Escreve_Jal = 6'd23;
parameter [5:0] Addi = 6'd24;
parameter [5:0] Addiu = 6'd25;
parameter [5:0] Add = 6'd26;
parameter [5:0] Sub = 6'd27;
parameter [5:0] And = 6'd28;
parameter [5:0] LoadStore = 6'd29;
parameter [5:0] Escreve_Arit_Ime = 6'd30;
parameter [5:0] Beqm = 6'd31;
parameter [5:0] Wait_Mem2 = 6'd32;
parameter [5:0] Wait_Ex1 = 6'd33;
parameter [5:0] Exc2 = 6'd34;
parameter [5:0] Escreve_Shift = 6'd35;
parameter [5:0] Escreve_Slt = 6'd36;
parameter [5:0] Overflow = 6'd37;
parameter [5:0] Escreve_Arit = 6'd38;
parameter [5:0] Lb = 6'd39;
parameter [5:0] Lh = 6'd40;
parameter [5:0] Lw = 6'd41;
parameter [5:0] Sb = 6'd42;
parameter [5:0] Sh = 6'd43;
parameter [5:0] Sw = 6'd44;
parameter [5:0] Pc_Set = 6'd45;
parameter [5:0] Wait_Store1 = 6'd46;
parameter [5:0] Wait_Calc_Branch = 6'd47;
parameter [5:0] Escreve_Slt_Ime = 6'd48;
parameter [5:0] Escreve_Mfh = 6'd49;
parameter [5:0] Load_Shift = 6'd50;
parameter [5:0] Load_Shift_V = 6'd51;
parameter [5:0] Wait_LS = 6'd52;
parameter [5:0] Escreve = 6'd53;
parameter [5:0] Wait_Store2 = 6'd54;
parameter [5:0] Wait_Jr = 6'd55;
parameter [5:0] Escreve_Branch = 6'd56;
parameter [5:0] Ex_DivZero = 6'd57;
parameter [5:0] Wait_Mem1 = 6'd58;
parameter [5:0] Wait_Ex2 = 6'd59;
parameter [5:0] RstDiv = 6'd60;

reg [5:0] estadoAtual = Reset;


always@ (posedge clk) begin	
	
	TestadoAtual = estadoAtual;
	
	if(rst ==1)begin
		estadoAtual = Reset;
	end
	
	case (estadoAtual)
		Reset: begin
			PcSel = 3'b00;
			Iord = 2'b00;
			StoreSel = 2'b00;
			LoadSel = 2'b00;
			ShiftSel = 3'b000;
			ScrA = 2'b00;
			ScrB = 2'b00;
			AluOp = 3'b000;
			PcWrite = 0;
			EpcWrite = 0;
			MemStatus = 0;
			IrWrite = 0;
			ScrShift = 0;
			RegWrite = 1;
			SignSel = 0;
			DivSel = 0;
			DivStart = 0;
			rstDiv = 1;
			LoadReg = 0; //tava 1
			//zerar pilha
			ScrReg = 2'b01;
			ScrData = 2'b00;
			
			estadoAtual = Busca;
		end
		
		Busca: begin
			PcWrite = 0;
			RegWrite = 0;
			EpcWrite = 1;
			Iord = 2'b00;
			MemStatus = 0;
			PcSel = 3'b000;
			ScrA = 2'b00;
			ScrB = 2'b00;
			AluOp = 3'b001;
			LoadReg = 1;
			DivStart = 0;
			
			estadoAtual = Wait_Mem1;
		end
		
		Wait_Mem1: begin
			EpcWrite = 0;
			estadoAtual = Wait_Ir;
		end
		
		Wait_Ir: begin
			PcWrite = 1;
			IrWrite = 1;
			estadoAtual = Wait_Calc_Branch;			
		end
		
		Wait_Calc_Branch: begin
			PcWrite = 0;
			estadoAtual = Calc_Branch;
		end
		
		Calc_Branch: begin
			ScrB = 2'b10;
			ScrA = 2'b00;
			AluOp = 3'b001;
			IrWrite = 0;
			
			case(Opcode)
				default: estadoAtual = OpIne;
				
				6'h0: begin
					case(Funct)
						default: estadoAtual = OpIne;
						6'h20: estadoAtual = Add;
						
						6'h24: estadoAtual = And;
						
						6'h1a: estadoAtual = RstDiv;
						
						6'h8: estadoAtual = Jr;
						
						6'h10: estadoAtual = Mfhi;
						
						6'h12: estadoAtual = Mflo;
						
						6'h0: estadoAtual = Load_Shift;
						
						6'h4: estadoAtual = Load_Shift_V;
						
						6'h2a: estadoAtual = Slt;
						
						6'h3: estadoAtual = Load_Shift;
						
						6'h7: estadoAtual = Load_Shift_V;
						
						6'h2: estadoAtual = Load_Shift;
						
						6'h22: estadoAtual = Sub;
	
						6'hd: estadoAtual = Break;
					endcase
				end
				
				6'h10: begin
					case (Funct)
						default: estadoAtual = OpIne;
						6'h10: estadoAtual = Rte;
					endcase
				end
				
				6'h8: estadoAtual = Addi;
				
				6'h9: estadoAtual = Addiu;
				
				6'h4: estadoAtual = Branch;
				
				6'h5: estadoAtual = Branch;

				6'h6: estadoAtual = Branch;
	
				6'h7: estadoAtual = Branch;
	
				6'h1: estadoAtual = Branch;
				
				6'h2c: estadoAtual = Beqm;
				
				6'h20: estadoAtual = LoadStore;
				
				6'h21: estadoAtual = LoadStore;
				
				6'hf: estadoAtual = Lui;
				
				6'h23: estadoAtual = LoadStore;
				
				6'h28: estadoAtual = LoadStore;
				
				6'h29: estadoAtual = LoadStore;
				
				6'ha: estadoAtual = Slti;
				
				6'h2b: estadoAtual = LoadStore;
				
				6'h2: estadoAtual = Jump;
				
				6'h3: estadoAtual = Jal;
			endcase
		end
		
		Rte: begin
			PcSel = 3'd2;
			PcWrite = 1;
			estadoAtual = Busca;
		end
		
		Jump: begin
			RegWrite = 0;
			PcSel = 3'd1;
			PcWrite = 1;
			estadoAtual = Busca;
		end
		
		Branch: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd1;
			AluOp = 3'd7;
			PcSel = 3'd0;
			LoadReg = 0;
			estadoAtual = Escreve_Branch;					
		end
		
		Escreve_Branch: begin
			case (Opcode)
				6'h4: if(ET == 1) PcWrite = 1;
				6'h5: if(ET == 0) PcWrite = 1;
				6'h6: if(GT == 0) PcWrite = 1;
				6'h7: if(GT == 1) PcWrite = 1;
				6'h1: if(LT == 0) PcWrite = 1;
				6'h2c: if(ET == 1) PcWrite = 1;
			endcase
		estadoAtual = Busca;
		end
		
		Lui: begin
			SignSel = 0;
			ScrReg = 2'd3;
			ScrData = 3'd5;
			RegWrite = 1;
			estadoAtual = Busca;
		end
		
		Jr: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd1;
			AluOp = 3'd1;
			estadoAtual = Wait_Jr;
		end
		
		Wait_Jr: begin
			PcSel = 3'd0;
			PcWrite = 1;
			estadoAtual = Busca;
		end
		
		RstDiv: begin
			rstDiv = 1;
			estadoAtual = Div;
		end

		Div: begin
			rstDiv = 0;
			RegWrite = 0;
			DivStart = 1;
			if(DivZero == 1) begin
				estadoAtual = Ex_DivZero;				
			end else if(ParaDiv == 1) begin
				estadoAtual = Busca;
			end else begin
				estadoAtual = Div;
			end
		end
		
		Add: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd1;
			AluOp = 3'd1;
			estadoAtual = Escreve_Arit;
		end
		
		Sub: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd1;
			AluOp = 3'd2;
			estadoAtual = Escreve_Arit;
		end
		
		And: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd1;
			AluOp = 3'd3;
			estadoAtual = Escreve_Arit;
		end
		
		Addi: begin
			RegWrite = 0;
			ScrA = 2'b01;
			ScrB = 2'b11;
			AluOp = 3'b001;
			SignSel = 0;
			estadoAtual = Escreve_Arit_Ime;
		end
		
		Addiu: begin
			RegWrite = 0;
			ScrA = 2'd1;
			ScrB = 2'd3;
			SignSel = 1;
			AluOp = 3'd1;
			estadoAtual = Escreve_Arit_Ime;
		end
		
		Escreve_Arit: begin		
			if(O == 1) begin
				estadoAtual = Overflow;
			end else begin
				RegWrite = 1;
				ScrReg = 2'd2;
				ScrData = 3'd6;
				estadoAtual = Busca;
			end
		end
		
		Escreve_Arit_Ime: begin
			if(O == 1) begin
				estadoAtual = Overflow;
			end else begin
				RegWrite = 1;
				ScrReg = 2'b11;
				ScrData = 3'b110;
				estadoAtual = Busca;
			end
		end
		
		Ex_DivZero: begin
			PcSel = 3'd5;
			PcWrite = 1;
			DivStart = 0;
			estadoAtual = Wait_Ex1;
		end
		
		Overflow: begin
			PcSel = 3'd4;
			PcWrite = 1;
			estadoAtual = Wait_Ex1;
		end
		
		Wait_Ex1: begin
			PcWrite = 0;
			Iord = 2'd0;
			estadoAtual = Wait_Ex2;
		end
		
		Wait_Ex2: begin
			estadoAtual = Exc2;
		end
		
		Exc2: begin
			PcSel = 3'd6;
			PcWrite = 1;
			estadoAtual = Busca;
		end
		
		OpIne: begin
			PcSel = 3'd3;
			PcWrite = 1;
			estadoAtual = Wait_Ex1;
		end
		
		Break: begin
			PcWrite = 0;
			estadoAtual = Break;
		end
		
		Sll: begin
			ShiftSel = 3'd2;
			estadoAtual = Escreve_Shift;
		end
		
		Sra: begin
			ShiftSel = 3'd4;
			estadoAtual = Escreve_Shift;
		end
		
		Srl: begin
			ShiftSel = 3'd3;
			estadoAtual = Escreve_Shift;
		end
		
		Sllv: begin
			ShiftSel = 3'd2;
			estadoAtual = Escreve_Shift;
		end
		
		Srav: begin
			ShiftSel = 3'd4;
			estadoAtual = Escreve_Shift;
		end
		
		Load_Shift: begin
			ShiftSel = 3'd1;
			RegWrite = 0;
			ScrShift = 1;
			case(Funct)
				6'h0: estadoAtual = Sll;
				6'h3: estadoAtual = Sra;
				6'h2: estadoAtual = Srl;
			endcase
		end
		
		Load_Shift_V: begin
			ShiftSel = 3'd1;
			RegWrite = 0;
			ScrShift = 0;
			case(Funct)
				6'h4: estadoAtual = Sllv;
				6'h7: estadoAtual = Srav;
			endcase
		end
		
		Escreve_Shift: begin
			RegWrite = 1;
			ScrData = 3'd2;
			ScrReg = 2'd2;
			ShiftSel = 3'd0;
			estadoAtual = Busca;
		end
		
		Jal: begin
			PcWrite = 1;
			RegWrite = 0;
			ScrA = 2'd0;
			AluOp = 3'd0;
			ScrReg = 2'd0;
			ScrData = 3'd6;
			PcSel = 3'd1;
			estadoAtual = Escreve_Jal;
		end
		
		Escreve_Jal: begin
			RegWrite = 1;
			estadoAtual = Busca;
		end
		
		Slt: begin
			RegWrite = 0;
			ScrA = 2'd1;
			AluOp = 3'd7;
			ScrB = 2'd1;
			estadoAtual = Escreve_Slt;
		end
		
		Escreve_Slt: begin
			RegWrite = 1;
			ScrReg = 2'd2;
			ScrData = 3'd4;
			estadoAtual = Busca;
		end
		
		Slti: begin
			RegWrite = 0;
			ScrA = 2'd1;
			AluOp = 3'd7;
			ScrB = 2'd3;
			estadoAtual = Escreve_Slt_Ime;
		end
		
		Escreve_Slt_Ime: begin
			RegWrite = 1;
			ScrReg = 2'd3;
			ScrData = 3'd4;
			estadoAtual = Busca;
		end
		
		Mfhi: begin
			ScrReg = 2'd2;
			ScrData = 3'd1;
			DivSel = 0;
			estadoAtual = Escreve_Mfh;
		end
		
		Mflo: begin
			ScrReg = 2'd2;
			ScrData = 3'd1;
			DivSel = 1;
			estadoAtual = Escreve_Mfh;
		end
		
		Escreve_Mfh: begin
			RegWrite = 1;
			estadoAtual = Busca;
		end
		
		LoadStore: begin
			RegWrite = 0;
			SignSel = 1;
			ScrA = 2'd1;
			ScrB = 2'd3;
			AluOp = 3'd1;
			Iord = 2'd1;
			MemStatus = 0;
			estadoAtual = Wait_Mem2;
		end
		
		Beqm: begin
			RegWrite = 0;
			Iord = 2'd2;
			MemStatus = 0;
			estadoAtual = Wait_Mem2;
		end
		
		Wait_Mem2: begin
			if (Opcode == 6'h2c) begin
				estadoAtual = Pc_Set;
				LoadReg = 0;
			end else begin
				estadoAtual = Wait_LS;
			end
		end
		
		Pc_Set: begin
			ScrA = 2'd2;
			ScrB = 2'd1;
			AluOp = 3'd7;
			PcSel = 3'd0;
			estadoAtual = Escreve_Branch;
		end
		
		Wait_LS: begin
			case (Opcode)
				6'h20: estadoAtual = Lb;
				
				6'h21: estadoAtual = Lh;
				
				6'h23: estadoAtual = Lw;
				
				6'h28: estadoAtual = Sb;
				
				6'h29: estadoAtual = Sh;
				
				6'h2b: estadoAtual = Sw;
			endcase
		end
		
		Lb: begin
			ScrReg = 2'd3;
			ScrData = 3'd3;
			LoadSel = 2'd0;
			estadoAtual = Escreve;
		end
		
		Lh: begin
			ScrReg = 2'd3;
			ScrData = 3'd3;
			LoadSel = 2'd1;
			estadoAtual = Escreve;
		end
		
		Lw: begin
			ScrReg = 2'd3;
			ScrData = 3'd3;
			LoadSel = 2'd2;
			estadoAtual = Escreve;
		end
		
		Sb: begin
			Iord = 2'd1;
			StoreSel = 2'd0;
			estadoAtual = Wait_Store1;
		end
		
		Sh: begin
			Iord = 2'd1;
			StoreSel = 2'd1;
			estadoAtual = Wait_Store1;
		end
		
		Sw: begin
			Iord = 2'd1;
			StoreSel = 2'd2;
			estadoAtual = Wait_Store1;
		end
		
		Wait_Store1: begin
			estadoAtual = Wait_Store2;
		end
		
		Wait_Store2: begin
			MemStatus = 1;
			estadoAtual = Busca;
		end
		
		Escreve: begin
			RegWrite = 1;
			estadoAtual = Busca;
		end
		
	endcase	
	
end
endmodule 