module Processador(

input rst,
input clk

//output wire [5:0] TOpcode,
//output wire [5:0] TFunct,
//output wire TDivZero,
//output wire TLT,
//output wire TGT,
//output wire TET,
//output wire TO,
//output wire TN,
//output wire TZ,
//output wire TParaDiv,
//output wire [2:0] TPcSel,
//output wire [1:0] TIord,
//output wire [1:0] TStoreSel,
//output wire [1:0] TLoadSel,
//output wire [1:0] TSrcReg,
//output wire [2:0] TSrcData,
//output wire [2:0] TShiftSel,
//output wire [1:0] TSrcA,
//output wire [1:0] TSrcB,
//output wire [2:0] TAluOp,
//output wire TPcWrite,
//output wire TEpcWrite,
//output wire TMemStatus,
//output wire TIrWrite,
//output wire TSrcShift,
//output wire TRegWrite,
//output wire TSignSel,
//output wire TDivSel,
//output wire TDivStart,
//output wire TrstDiv,
//output wire TLoadReg,
//
//output wire [31:0] TOut_Pc,
//output wire [31:0] TOut_Epc,
//output wire [31:0] TOut_StoreBox,
//output wire [31:0] TOut_LoadBox,
//output wire [31:0] TOut_Mem
//output wire [31:0] TOut_MemReg,
//output wire [31:0] TOut_ShiftReg,
//output wire [31:0] TOut_Div,
//output wire [31:0] TOut_A,
//output wire [31:0] TOut_B,
//output wire [31:0] TOut_Alu,
//output wire [31:0] TMem_Ex,
//output wire [31:0] TJump_Ex
//output wire [31:0] TImmediate_Ex,
//output wire [31:0] TLT_Ex,
//output wire [31:0] TFio_Branch,
//output wire [31:0] TFio_Lui,
//output wire [4:0] TRS,
//output wire [4:0] TRT,
//output wire [4:0] TRD,
//output wire [4:0] TSA,
//output wire [25:0] TJump,
//output wire [15:0] TImmediate,
//output wire [31:0] TAlu_Result,
//output wire [31:0] Tin_A,
//output wire [31:0] Tin_B,
//
//output wire [31:0] TOut_PcSel,
//output wire [31:0] TOut_Iord
//output wire [4:0] TOut_SrcReg,
//output wire [31:0] TOut_SrcData
//output wire [31:0] TOut_Shift_in,
//output wire [4:0] TOut_Shift_n
//output wire [31:0] TOut_SrcA,
//output wire [31:0] TOut_SrcB
//output wire [5:0] TestadoAtual
);

//fios do controle
wire [5:0] Opcode;
wire [5:0] Funct;
wire DivZero;
wire LT;
wire GT;
wire ET;
wire O;
wire N;
wire Z;
wire ParaDiv;
wire [2:0] PcSel;
wire [1:0] Iord;
wire [1:0] StoreSel;
wire [1:0] LoadSel;
wire [1:0] SrcReg;
wire [2:0] SrcData;
wire [2:0] ShiftSel;
wire [1:0] SrcA;
wire [1:0] SrcB;
wire [2:0] AluOp;
wire PcWrite;
wire EpcWrite;
wire MemStatus;
wire IrWrite;
wire SrcShift;
wire RegWrite;
wire SignSel;
wire DivSel;
wire DivStart;
wire rstDiv;
wire LoadReg;
wire estadoAtual;

//fios da unidade de processamento
wire [31:0] Out_Pc;
wire [31:0] Out_Epc;
wire [31:0] Out_StoreBox;
wire [31:0] Out_LoadBox;
wire [31:0] Out_Mem;
wire [31:0] Out_MemReg;
wire [31:0] Out_ShiftReg;
wire [31:0] Out_Div;
wire [31:0] Out_A;
wire [31:0] Out_B;
wire [31:0] Out_Alu;
wire [31:0] Mem_Ex;
wire [31:0] Jump_Ex;
wire [31:0] Immediate_Ex;
wire [31:0] LT_Ex;
wire [31:0] Fio_Branch;
wire [31:0] Fio_Lui;
wire [4:0] RS;
wire [4:0] RT;
wire [4:0] RD;
wire [4:0] SA;
wire [25:0] Jump;
wire [15:0] Immediate;
wire [31:0] Alu_Result;
wire [31:0] in_A;
wire [31:0] in_B;

//fios dos muxes
wire [31:0] Out_PcSel;
wire [31:0] Out_Iord;
wire [4:0] Out_SrcReg;
wire [31:0] Out_SrcData;
wire [31:0] Out_Shift_in;
wire [4:0] Out_Shift_n;
wire [31:0] Out_SrcA;
wire [31:0] Out_SrcB;

//ligando os fios aos modulos
Banco_reg bancoRegs(clk, rst, RegWrite, RS, RT, Out_SrcReg, Out_SrcData, in_A, in_B);
Controle controle(clk, rst, Opcode, Funct, DivZero, LT, GT, ET, O, ParaDiv,
				PcSel, Iord, StoreSel, LoadSel, SrcReg, SrcData, ShiftSel, SrcA, SrcB,
				AluOp, PcWrite, EpcWrite, MemStatus, IrWrite, SrcShift, RegWrite, SignSel,
				DivSel, DivStart, rstDiv, LoadReg, estadoAtual);
Div divisor(Out_A, Out_B, DivStart, ParaDiv, DivSel, clk, rstDiv, Out_Div, DivZero);
Extends8_32 e1(Out_Mem, Mem_Ex);
Extends26_32 e2(Jump, Out_Pc, Jump_Ex);
Extends1_32 e3(LT, LT_Ex);
Extends16_32 e4(SignSel, Immediate, Immediate_Ex);
IR instReg(clk, rst, IrWrite, Out_Mem, Opcode, RS, RT, RD, SA, Funct, Immediate, Jump);
LoadBox loadbox(rst, LoadSel, Out_MemReg, Out_LoadBox);
StoreBox storebox(clk, rst, StoreSel, Out_MemReg, Out_B, Out_StoreBox);
Memoria memoria(Out_Iord, clk, MemStatus, Out_StoreBox, Out_Mem);
mux_a mux_A(SrcA, Out_Pc, Out_A, Out_Mem, Out_SrcA);
mux_b mux_B(SrcB, Out_B, Fio_Branch, Immediate_Ex, Out_SrcB);
mux_data_in mux_DataIn(SrcData, Out_Div, Out_ShiftReg, Out_LoadBox, LT_Ex, Fio_Lui,
					Out_Alu, Out_SrcData);
mux_iord mux_Iord(Iord, Out_Pc, Out_Alu, Out_A, Out_Iord);
mux_shift_in mux_ShiftIn(SrcShift, Out_B, Out_A, Out_Shift_in);
mux_shift_n mux_ShiftN(SrcShift, Out_B, SA, Out_Shift_n);
mux_src_reg mux_SrcReg(SrcReg, RT, RD, Out_SrcReg);
mux_pc mux_Pc(PcSel, Out_Alu, Jump_Ex, Out_Epc, Mem_Ex, Out_PcSel);
RegDesloc shiftReg(clk, rst, ShiftSel, Out_Shift_n, Out_Shift_in, Out_ShiftReg);
Registrador regA(clk, rst, LoadReg, in_A, Out_A);
Registrador regB(clk, rst, LoadReg, in_B, Out_B);
Registrador memReg(clk, rst, LoadReg, Out_Mem, Out_MemReg);
Registrador aluOut(clk, rst, LoadReg, Alu_Result, Out_Alu);
Registrador pc(clk, rst, PcWrite, Out_PcSel, Out_Pc);
Registrador epc(clk, rst, EpcWrite, Out_Pc, Out_Epc);
SLL16 shift_left_16(Immediate_Ex, Fio_Lui);
SLL2 shift_left_2(Immediate_Ex, Fio_Branch);
ula32 ula(Out_SrcA, Out_SrcB, AluOp, Alu_Result, O, N, Z, ET, GT, LT);

//resultado dos testes
//assign TestadoAtual = estadoAtual;
//assign TOpcode = Opcode;
//assign TFunct = Funct;
//assign TDivZero = DivZero;
//assign TET = ET;
//assign TO = O;
//assign TN = N;
//assign TZ = Z;
//assign TParaDiv = ParaDiv;
//assign TPcSel = PcSel;
//assign TIord = Iord;
//assign TStoreSel = StoreSel;
//assign TLoadSel = LoadSel;
//assign TSrcReg = SrcReg;
//assign TSrcData = SrcData;
//assign TShiftSel = ShiftSel;
//assign TSrcA = SrcA;
//assign TSrcB = SrcB;
//assign TAluOp = AluOp;
//assign TPcWrite = PcWrite;
//assign TEpcWrite = EpcWrite;
//assign TMemStatus = MemStatus;
//assign TIrWrite = IrWrite;
//assign TSrcShift = SrcShift;
//assign TRegWrite = RegWrite;
//assign TSignSel = SignSel;
//assign TDivSel = DivSel;
//assign TDivStart = DivStart ;
//assign TrstDiv = rstDiv;
//assign TLoadReg = LoadReg;
//assign TOut_Pc = Out_Pc;
//assign TOut_Epc = Out_Epc;
//assign TOut_StoreBox = Out_StoreBox;
//assign  TOut_LoadBox = Out_LoadBox;
//assign  TOut_Mem = Out_Mem;
//assign  TOut_MemReg = Out_MemReg;
//assign  TOut_ShiftReg = Out_ShiftReg;
//assign  TOut_Div = Out_Div;
//assign  TOut_A = Out_A;
//assign  TOut_B = Out_B;
//assign  TOut_Alu = Out_Alu;
//assign  TMem_Ex = Mem_Ex;
//assign  TJump_Ex = Jump_Ex;
//assign  TImmediate_Ex = Immediate_Ex;
//assign  TLT_Ex = LT_Ex;
//assign  TFio_Branch = Fio_Branch;
//assign  TFio_Lui = Fio_Lui;
//assign  TRS = RS;
//assign  TRT = RT;
//assign  TRD = RD;
//assign  TSA = SA;
//assign  TJump = Jump;
//assign  TImmediate =Immediate;
//assign  TAlu_Result = Alu_Result;
//assign  Tin_A = in_A;
//assign  Tin_B = in_B;
//
//assign  TOut_PcSel = Out_PcSel;
//assign  TOut_Iord = Out_Iord;
//assign  TOut_SrcReg = Out_SrcReg;
//assign  TOut_SrcData = Out_SrcData;
//assign  TOut_Shift_in = Out_Shift_in;
//assign  TOut_Shift_n = Out_Shift_n;
//assign  TOut_SrcA = Out_SrcA;
//assign  TOut_SrcB = Out_SrcB;

endmodule 